import { Api, Resource, Field } from "@api-platform/api-doc-parser";
import path from "path";
import { fileURLToPath } from "url";
import fs from "fs";
import tmp from "tmp";
import TypescriptInterfaceGenerator from "./TypescriptInterfaceGenerator.js";
var dirname = path.dirname(fileURLToPath(import.meta.url));
test("Generate a typescript interface", function () {
  var generator = new TypescriptInterfaceGenerator({
    templateDirectory: "".concat(dirname, "/../../templates")
  });
  var tmpobj = tmp.dirSync({
    unsafeCleanup: true
  });
  var resource = new Resource("abc", "http://example.com/foos", {
    id: "foo",
    title: "Foo",
    readableFields: [new Field("bar", {
      id: "http://schema.org/url",
      range: "http://www.w3.org/2001/XMLSchema#string",
      reference: null,
      required: true,
      description: "An URL"
    })],
    writableFields: [new Field("foo", {
      id: "http://schema.org/url",
      range: "http://www.w3.org/2001/XMLSchema#datetime",
      reference: null,
      required: true,
      description: "An URL"
    }), new Field("foobar", {
      id: "http://schema.org/url",
      range: undefined,
      reference: new Resource("foobar", "http://example.com/FooBar", {
        title: "FooBar"
      }),
      required: false
    })]
  });
  var api = new Api("http://example.com", {
    entrypoint: "http://example.com:8080",
    title: "My API",
    resources: [resource]
  });
  generator.generate(api, resource, tmpobj.name);
  expect(fs.existsSync(tmpobj.name + "/interfaces/foo.ts")).toBe(true);
  var res = "export interface Foo {\n  \"@id\"?: string;\n  foo: any;\n  foobar?: string[];\n  readonly bar: string;\n}\n";
  expect(fs.readFileSync(tmpobj.name + "/interfaces/foo.ts").toString()).toEqual(res);
  tmpobj.removeCallback();
});
test("Generate a typescript interface without references to other interfaces", function () {
  var generator = new TypescriptInterfaceGenerator({
    templateDirectory: "".concat(dirname, "/../../templates")
  });
  var tmpobj = tmp.dirSync({
    unsafeCleanup: true
  });
  var resource = new Resource("abc", "http://example.com/foos", {
    id: "foo",
    title: "Foo",
    readableFields: [new Field("bar", {
      id: "http://schema.org/url",
      range: "http://www.w3.org/2001/XMLSchema#string",
      reference: null,
      required: true,
      description: "An URL"
    })],
    writableFields: [new Field("foo", {
      id: "http://schema.org/url",
      range: "http://www.w3.org/2001/XMLSchema#datetime",
      reference: null,
      required: true,
      description: "An URL"
    })]
  });
  var api = new Api("http://example.com", {
    entrypoint: "http://example.com:8080",
    title: "My API",
    resources: [resource]
  });
  generator.generate(api, resource, tmpobj.name);
  expect(fs.existsSync(tmpobj.name + "/interfaces/foo.ts")).toBe(true);
  var res = "export interface Foo {\n  \"@id\"?: string;\n  foo: any;\n  readonly bar: string;\n}\n";
  expect(fs.readFileSync(tmpobj.name + "/interfaces/foo.ts").toString()).toEqual(res);
  tmpobj.removeCallback();
});
test("Generate a typescript interface with an explicit id field in the readableFields", function () {
  var generator = new TypescriptInterfaceGenerator({
    templateDirectory: "".concat(dirname, "/../../templates")
  });
  var tmpobj = tmp.dirSync({
    unsafeCleanup: true
  });
  var resource = new Resource("abc", "http://example.com/foos", {
    id: "foo",
    title: "Foo",
    readableFields: [new Field("bar", {
      id: "http://schema.org/url",
      range: "http://www.w3.org/2001/XMLSchema#string",
      reference: null,
      required: true,
      description: "An URL"
    }), new Field("id", {
      id: "http://schema.org/url",
      range: "http://www.w3.org/2001/XMLSchema#string",
      reference: null,
      required: false,
      description: "Id"
    })],
    writableFields: [new Field("foo", {
      id: "http://schema.org/url",
      range: "http://www.w3.org/2001/XMLSchema#datetime",
      reference: null,
      required: true,
      description: "An URL"
    })]
  });
  var api = new Api("http://example.com", {
    entrypoint: "http://example.com:8080",
    title: "My API",
    resources: [resource]
  });
  generator.generate(api, resource, tmpobj.name);
  expect(fs.existsSync(tmpobj.name + "/interfaces/foo.ts")).toBe(true);
  var res = "export interface Foo {\n  \"@id\"?: string;\n  foo: any;\n  readonly bar: string;\n  readonly id?: string;\n}\n";
  expect(fs.readFileSync(tmpobj.name + "/interfaces/foo.ts").toString()).toEqual(res);
  tmpobj.removeCallback();
});
test("Generate a typescript interface with invalid JS identifiers for fields and title", function () {
  var generator = new TypescriptInterfaceGenerator({
    templateDirectory: "".concat(dirname, "/../../templates")
  });
  var tmpobj = tmp.dirSync({
    unsafeCleanup: true
  });
  var resource = new Resource("abc", "http://example.com/foos", {
    id: "foo",
    title: "Foo-foo",
    readableFields: [new Field("bar-bar", {
      id: "http://schema.org/url",
      range: "http://www.w3.org/2001/XMLSchema#string",
      reference: null,
      required: true,
      description: "An URL"
    }), new Field("id", {
      id: "http://schema.org/url",
      range: "http://www.w3.org/2001/XMLSchema#string",
      reference: null,
      required: false,
      description: "Id"
    })],
    writableFields: [new Field("foo-foo", {
      id: "http://schema.org/url",
      range: "http://www.w3.org/2001/XMLSchema#datetime",
      reference: null,
      required: true,
      description: "An URL"
    })]
  });
  var api = new Api("http://example.com", {
    entrypoint: "http://example.com:8080",
    title: "My API",
    resources: [resource]
  });
  generator.generate(api, resource, tmpobj.name);
  expect(fs.existsSync(tmpobj.name + "/interfaces/foo-foo.ts")).toBe(true);
  var res = "export interface Foo_foo {\n  \"@id\"?: string;\n  \"foo-foo\": any;\n  readonly \"bar-bar\": string;\n  readonly id?: string;\n}\n";
  expect(fs.readFileSync(tmpobj.name + "/interfaces/foo-foo.ts").toString()).toEqual(res);
  tmpobj.removeCallback();
});